/**
 * Copyright 2007-2008 Bagotios
 */

class AdvancedScoreboardMutator extends UTMutator config(AdvancedScoreBoardV06);

var array<ASUTLPRI> PRIs;
var array<UTPlayerController> UTPCs;
var config bool bUseAdvancedScoreboard;
var config bool bAnnounceMultiKills;
var config int MaxNameChanges;
var config int AnnouncerLevel;

/**	add a new scoreboard for each PlayerController only once!
 *	Doesn't look like you can replace the scoreboard before the first pawn is spawned.
 *	If replaced when the PlayerController is seen the game overrides with default scoreboard
 */
function ModifyPlayer(Pawn Other)
{
	local UTPlayerController UTPC;
	if (Other!=None) 
	{
		UTPC = UTPlayerController (Other.Owner);
		if (UTPC != None && bUseAdvancedScoreboard && UTPCs.Find(UTPC) == -1)
		{
			//LogInternal("Gametype: " $WorldInfo.GetGameClass().name);
			if (string(WorldInfo.GetGameClass().name) == "UTCTFGame_Content")
			{
				UTPC.ClientSetHUD (Class'AdvancedUTCTFHUD', Worldinfo.Game.Scoreboardtype);
			}
			if (string(WorldInfo.GetGameClass().name) == "UTTeamGame")
			{
				UTPC.ClientSetHUD (Class'AdvancedUTTDMHUD', Worldinfo.Game.Scoreboardtype);
			}
			if (string(WorldInfo.GetGameClass().name) == "UTDeathmatch")
			{
				UTPC.ClientSetHUD (Class'AdvancedUTHUD', Worldinfo.Game.Scoreboardtype);
			}
			if (string(WorldInfo.GetGameClass().name) == "UTVehicleCTFGame_Content")
			{
				UTPC.ClientSetHUD (Class'AdvancedUTVehicleCTFHUD', Worldinfo.Game.Scoreboardtype);
			} 
			if (string(WorldInfo.GetGameClass().name) == "UTOnslaughtGame_Content")
			{
				UTPC.ClientSetHUD (Class'AdvancedUTOnslaughtHUD', Worldinfo.Game.Scoreboardtype);
			}
			if (string(WorldInfo.GetGameClass().name) == "UTDuelGame")
			{
				UTPC.ClientSetHUD (Class'AdvancedUTDuelHUD', Worldinfo.Game.Scoreboardtype);
			} 	 		  	 	
			UTPCs.addItem(UTPC);
		}
	}
 	Super.ModifyPlayer(Other);

}

function string GetStringToMKLvl(int MKLevel)
{
	local string toReturn;
	switch(MKLevel)
	{
		case 1: toReturn = "Double Kill"; break;
		case 2: toReturn = "Multi Kill"; break;
		case 3: toReturn = "Mega Kill"; break;
		case 4: toReturn = "Ultra Kill"; break;
		default: toReturn = "Monster Kill!"; break;
	}

		return toReturn;
}

function Tick(float DeltaTime)
{
	local ASUTLPRI temp;
	local ASUTLPRI Receiver;
	local UTPlayerController UTPC;

	foreach PRIs(temp)
	{
		if(bAnnounceMultiKills)
		{
			if(temp == None)
				PRIs.removeItem(temp);
			else if(temp.UTPRI == None)
				PRIs.removeItem(temp);
			else
			{
				if(temp.LastKillCount < temp.UTPRI.Kills && temp.UTPRI.MultiKillLevel > 0 && temp.UTPRI.MultiKillLevel >= AnnouncerLevel)
				{
					temp.LastKillCount = temp.UTPRI.Kills;
					foreach PRIs(Receiver)
					{		
						UTPC = UTPlayerController(Receiver.UTPRI.Owner);
						if(UTPC != None)
						{
							UTPC.ClientMessage(temp.UTPRI.GetPlayerAlias()  $ " had a " $ GetStringToMKLvl(temp.UTPRI.MultiKillLevel));
						}
					}
				}
			}
		}
		if(temp == None)
			PRIs.removeItem(None);
	}
}
 
//We don't want to replace anything, we rather want to add something to UTPlayerReplicationInfos
function bool CheckReplacement(Actor Other)
{
	local UTLinkedReplicationInfo UTLRI;
	local bool bFound;
	local ASUTLPRI newRI;
	
	//if we found a ReplicationInfo we add ours LinkedReplicationInfo and all LinkedReplicationInfos stored so far
	//get added behind ours
	if(Other != None)
	{	
		if(Other.IsA('UTPlayerReplicationInfo'))
		{
			UTLRI = UTPlayerReplicationInfo(Other).CustomReplicationInfo;
			while(UTLRI != None)
			{
				if(UTLRI.IsA('ASUTLPRI'))
				{	
					bFound = true;
					break;
				}
				UTLRI = UTLRI.NextReplicationInfo;
			}
			if(bFound)
			{
				LogInternal("ASUTLPRI already spawned");
				return true;
			}
			else
			{
				if(string(WorldInfo.GetGameClass().name) == "UTCTFGame_Content" || string(WorldInfo.GetGameClass().name) == "UTVehicleCTFGame_Content")
					newRI = Spawn(class'CTFASUTLPRI', UTPlayerReplicationInfo(Other));
				else if(string(WorldInfo.GetGameClass().name) == "UTOnslaughtGame_Content")
					newRI = Spawn(class'ONSASUTLPRI', UTPlayerReplicationInfo(Other));
				else
					newRI = Spawn(class'DMASUTLPRI', UTPlayerReplicationInfo(Other));
				newRI.init(MaxNameChanges);
				newRI.NextReplicationInfo = UTPlayerReplicationInfo(Other).CustomReplicationInfo;
				UTPlayerReplicationInfo(Other).CustomReplicationInfo = newRI;
				PRIs.addItem(newRI);
				
				if(UTPlayerReplicationInfo(Other).CustomReplicationInfo.IsA('ASUTLPRI'))
				{
					LogInternal("Spawning of ASUTLPRI Successful");
				}
				else
					LogInternal("Spawning of ASUTLPRI Failed");
			}
			return true;
		}
	}
	return true;
}

function Mutate(string MutateString, PlayerController Sender)
{
	local string cmd;

	
	cmd = Caps(MutateString);
	
	if(cmd == "TOGGLEADVANCEDSCOREBOARD" && Sender.PlayerReplicationInfo.bAdmin)
	{
		ToggleScoreBoardUsage(Sender);
		SaveConfig();
	}
	else if(cmd == "TOGGLEMULTIKILLANNOUNCER" && Sender.PlayerReplicationInfo.bAdmin)
	{
		ToggleMultiKillAnnouncer(Sender);
		SaveConfig();
	}
	else if(Left(cmd, 7) == "SETNAME")
	{
		SetCustomName(MutateString, Sender);
	}
	else if(cmd == "LISTNAMES")
	{
		ListNames(Sender);
	}
	else if(cmd == "RESETNAME")
	{
		ResetName(Sender);	
	}
	else if(cmd == "RESETALLNAMES")
	{
		ResetAllNames();
	}
	
	
	Super.Mutate(MutateString, Sender);
}

function ResetAllNames()
{
	local PlayerController PC;
	
	foreach dynamicactors(class'PlayerController', PC)
	{
		ResetName(PC);
	}
}

function ResetName(PlayerController Sender)
{
	local UTLinkedReplicationInfo UTLRI;
	if (Sender.PlayerReplicationInfo != None)
	{
		UTLRI = UTPlayerReplicationInfo(Sender.PlayerReplicationInfo).CustomReplicationInfo;
		while(UTLRI != None)
		{	
			if(UTLRI.IsA('ASUTLPRI'))
			{
				ASUTLPRI(UTLRI).ResetName();
			}
			UTLRI = UTLRI.NextReplicationInfo;
		}
	}
}

function ListNames(PlayerController Sender)
{
	local UTPlayerReplicationInfo Player;
	
	Sender.ClientMessage("PlayerAlias = LoginName");
	Sender.ClientMessage("-----------------------");
	
	foreach dynamicactors(class'UTPlayerReplicationInfo', Player)
	{	
		Sender.ClientMessage(Player.getPlayerAlias() $ " = " $ Player.Playername);
	}
}

function SetCustomName(string MutateString, PlayerController Sender)
{
	local string customName;
	local UTLinkedReplicationInfo UTLRI;
	local UTPlayerReplicationInfo PRI;
	local bool bUnique;
	bUnique = true;
	customName = Repl(MutateString, "SETNAME ", "", false);

	foreach dynamicactors(class'UTPlayerReplicationInfo', PRI)
	{
		if(PRI.getPlayerAlias() == customName)
			bUnique = false;
	}
	
	if (Sender.PlayerReplicationInfo != None && bUnique)
	{
		UTLRI = UTPlayerReplicationInfo(Sender.PlayerReplicationInfo).CustomReplicationInfo;
		while(UTLRI != None)
		{	
			if(UTLRI.IsA('ASUTLPRI'))
			{
				ASUTLPRI(UTLRI).SetCustomName(customName);
			}
			UTLRI = UTLRI.NextReplicationInfo;
		}
	}
	else if(!bUnique)
	{
		Sender.ClientMessage("Cannot use a name that is already in use!");
	}
}
function ToggleScoreBoardUsage(PlayerController Sender)
{
	bUseAdvancedScoreboard = !bUseAdvancedScoreBoard;
	if(bUseAdvancedScoreBoard)
		Sender.ClientMessage("AdvancedScoreBoard has been activated");
	else
		Sender.ClientMessage("AdvancedScoreBoard has been deactivated");

	Sender.ClientMessage("This will effect all players joining after the command has been issued");
}

function ToggleMultiKillAnnouncer(PlayerController Sender)
{
	bAnnounceMultiKills = !bAnnounceMultiKills;
	if(bAnnounceMultiKills)
		Sender.ClientMessage("Multi kills will now be announced");
	else
		Sender.ClientMessage("Multi kills will no longer be announced");
}

function GetSeamlessTravelActorList(bool bToEntry, out array<Actor> ActorList)
{
  local ASUTLPRI LRI;
   foreach dynamicactors(class'ASUTLPRI', LRI)
   {
      ActorList[ActorList.length] = LRI;
      LRI.Reset();
   }

   super.GetSeamlessTravelActorList(bToEntry,ActorList);
}

defaultproperties
{
	bUseAdvancedScoreboard = false;
	bAnnounceMultiKills = true;
	MaxNameChanges = 3;
	AnnouncerLevel = 0;
}