/**
 * Copyright 2007-2008 Bagotios
 */

class AdvancedUTCTFScoreboardPanel extends UTCTFScoreboardPanel;

//clantag type
var int CTType;
var bool bDisplayAdditionalInformation;

/** Sets the header strings using localized values
 * Assumption: Only called once per map, right point to init our ClientData
 */
function SetHeaderStrings()
{
	HeaderTitle_Name = Localize( "Scoreboards", "Name", "UTGameUI" );
	//HeaderTitle_Score = Localize( "Scoreboards", "Score", "UTGameUI" );
	HeaderTitle_Score = "Points/Kills";
}

/**
* Draw the Player's Score
*/
function float DrawScore(UTPlayerReplicationInfo PRI, float YPos, int FontIndex, float FontScale)
{
	local string Spot;
	local float Width, Height;
	local ASUTLPRI LRI;
	LRI = GetLRI(PRI);
	
	// Draw the player's Kills
	if(LRI != none)
		Spot = GetPlayerScore(PRI)$"/"$LRI.PureKills;
	else
		Spot = GetPlayerScore(PRI);
	Canvas.Font = Fonts[FontIndex].Font;
	Canvas.StrLen(Spot, Width, Height);
	DrawString( Spot, RightColumnPosX+RightColumnWidth-Width, YPos,FontIndex,FontScale);

	return RightColumnPosX;
}

/**
 * Update the LRI of a PRI
 */
function ASUTLPRI GetLRI(UTPlayerReplicationInfo PRI)
{
	//The search should go fast because usually there are not that many CustomPRIs attached
	local UTLinkedReplicationInfo UTLRI;
	
	if (PRI != None)
	{
		UTLRI = PRI.CustomReplicationInfo;
		while(UTLRI != None)
		{	
			if(UTLRI.IsA('ASUTLPRI'))
			{
				return (ASUTLPRI(UTLRI));
			}
			UTLRI = UTLRI.NextReplicationInfo;
		}
	}
	
	return None;
}

/**
 * @Overwrite
 * Add Deaths and Effi to the standard information.
 */
function string GetRightMisc(UTPlayerReplicationInfo PRI)
{
	local int Deaths, Efficiency;
	local string BaseString;
	local ASUTLPRI LRI;
	LRI = GetLRI(PRI);

	if(PRI.WorldInfo.NetMode != NM_Standalone && LRI != None) 
	{
		BaseString = Super.GetRightMisc(PRI);
		Deaths = int(GetPlayerDeaths(PRI));


		if(LRI.PureKills != 0)
			Efficiency = (float(LRI.PureKills)/(float(LRI.PureKills)+(float(GetPlayerDeaths(PRI)))))*100;
		else
			Efficiency = 0;
		//PPH = int(float(Score) / (float(TotalSeconds+1)/float(3600)));

		return BaseString$"   "$"Deaths "$Deaths$"  "$"Effi "$Efficiency$"%";
	}
	return "";
}

/**
 * Add more information
 */
function string GetLowerRightMisc(UTPlayerReplicationInfo PRI)
{
	local CTFASUTLPRI LRI;
	//cast to the correct form of ASUTLPRI
	LRI = CTFASUTLPRI(GetLRI(PRI));

	if(PRI.WorldInfo.NetMode != NM_Standalone && LRI != None) 
	{
		return "EFC Kills: " $ LRI.FlagCarrierKills $ "  Flag Touches: " $ LRI.FlagTouches $ "  Caps: " $ LRI.FlagCaptures $ "  Returns: " $ LRI.FlagReturns;
	}
	return "";
}

/**
 * Draw the Player's Name
 */
function DrawPlayerName(UTPlayerReplicationInfo PRI, float NameOfst, float NameClipX, out float YPos, int FontIndex, float FontScale, bool bIncludeClan)
{

	local float XL, YL;
	local string Spot;
	local Color c ; 
	c = Canvas.DrawColor;
	
	if(PRI.bAdmin)
		Canvas.DrawColor = class'UTHUD'.default.GreenColor;
	Spot = bIncludeClan ? GetPlayerNameStr(PRI) : GetClanTagStr(PRI)$GetPlayerNameStr(PRI);
	StrLen(Spot, XL, YL, FontIndex, FontScale * MainPerc);
	YL = Fonts[FontIndex].CharHeight * FontScale * MainPerc;

	if ( XL > (NameClipX - NameOfst) && !bIncludeClan )
	{
		Spot = GetPlayerNameStr(PRI);
	}

	DrawString( Spot, NameOfst, YPos, FontIndex, FontScale * MainPerc);

	YPos += YL;
	Canvas.DrawColor = c;
}

/**
 * Returns the Clan Tag in the PRI
 * @Overwrite
 * remove those annoying brackets
 */
function string GetClanTagStr(UTPlayerReplicationInfo PRI)
{
	local UTGameReplicationInfo GRI;
	
	if ( PRI != none )
	{
		GRI = UTGameReplicationInfo(PRI.WorldInfo.GRI);
		if ( GRI != none && GRI.bStoryMode )
		{
			if (PRI.bBot || PRI.SinglePlayerCharacterIndex == INDEX_NONE)
			{
				return "";
			}

			return "["$GRI.SinglePlayerBotNames[PRI.SinglePlayerCharacterIndex]$"]";
		}
		if(CTType == 0)
			return PRI.ClanTag != "" ? "[" $(PRI.ClanTag) $ "]" : "";
		else if(CTType == 1)
			return PRI.ClanTag != "" ? (PRI.ClanTag) : "";
		else
			return "";
	}
	else
	{
		return "[Clan]";
	}
}

/**
 * Returns the Player's Name and if the user wants the clantag
 */
function string GetPlayerNameStr(UTPlayerReplicationInfo PRI)
{
	
	if(PRI != None)
	{
		if(CTType != 2)
			return PRI.GetPlayerAlias();
		else
			return PRI.ClanTag $ PRI.GetPlayerAlias();
	}
	else
	{
		return FakeNames[NameCnt];
	}
}

/**
 * Draw any Misc data
 */
function DrawMisc(UTPlayerReplicationInfo PRI, float NameOfst, out float YPos, int FontIndex, float FontScale)
{
	local string Spot;
	local float XL,YL;
	
	if(bDisplayAdditionalInformation)
	{
		// Draw the Misc Strings
		if ( FontIndex < 0 )
		{
			return;
		}
	
		YPos += PlayerNamePad * ResolutionScale;
		DrawString( GetLeftMisc(PRI), NameOfst, YPos - MiscPosAdjust, FontIndex, FontScale);
		Spot = GetRightMisc(PRI);
		StrLen(Spot,XL,YL, FontIndex, FontScale);
		DrawString( Spot, Canvas.ClipX-XL-15, YPos - MiscPosAdjust, FontIndex,FontScale);
		YPos += Fonts[FontIndex].CharHeight * FontScale;
		Spot = GetLowerRightMisc(PRI);
		StrLen(Spot,XL,YL, FontIndex, FontScale);
		DrawString( Spot, Canvas.ClipX-XL-15, YPos - MiscPosAdjust, FontIndex,FontScale);
		YPos += Fonts[FontIndex].CharHeight * FontScale - MiscPosAdjust;
	}
	else
		Super.DrawMisc(PRI, NameOfst, YPos, FontIndex, FontScale);
	
}

event PostInitialize()
{
	Super.PostInitialize();

	if(bDisplayAdditionalInformation)
	{
		HighlightPad = 6.0f;
		PlayerNamePad = 1.0f;
	}
	if(UTHudSceneOwner.GetPlayerHud().IsA('AdvancedUTVehicleCTFHUD'))
	{
		bDisplayAdditionalInformation = (AdvancedUTVehicleCTFHUD(UTHudSceneOwner.GetPlayerHud())).bDisplayAdditionalInformation;
		CTType = (AdvancedUTVehicleCTFHUD(UTHudSceneOwner.GetPlayerHud())).CTType;
	}
	else
	{
		bDisplayAdditionalInformation = (AdvancedUTCTFHUD(UTHudSceneOwner.GetPlayerHud())).bDisplayAdditionalInformation;
		CTType = (AdvancedUTCTFHUD(UTHudSceneOwner.GetPlayerHud())).CTType;
	}
}


/**
 * This function needs to be overwritten because we use 2 rows of right misc information
 * Figure a way to fit the data.  This will probably be specific to each game type
 *
 * @Param	GRI 				The Game ReplicationIfno
 * @Param	FontIndex			The Index to use for the main text
 * @Param 	ClanTagFontIndex	The Index to use for the Clan Tag
 * @Param	MiscFontIndex		The Index to use for the Misc tag
 * @Param	FontSCale			The final font scaling factor to use if all else fails
 * @Param	bPrimeList			Should only be true the first call.  Will build a list of
 *								who needs to be checked.
 */
function float AutoFit(UTGameReplicationInfo GRI, out int FontIndex,out int ClanTagFontIndex,
					out int MiscFontIndex, out float FontScale, bool bPrimeList)
{
	local float CellHeight;
	local bool bRecurse;
	local int CellHeightHack;
		
	if(bDisplayAdditionalInformation)
	{
		// We need to prime our list, so do that first.
		if ( bPrimeList )
		{
			if ( UTHudSceneOwner.IsGame() )
			{
				GetPRIList(GRI);
	
				if (bInteractive)
				{
					if (SelectedPI != INDEX_None )
					{
						CheckSelectedPRI();
					}
				}
			}
			else
			{
				// Create Fake Entries for the editor
				PRIList.Length = EditorTestNoPlayers;
			}
		}

		// Calculate the Actual Cell Height given all the data
		CellHeight  = Fonts[FontIndex].CharHeight * MainPerc * FontScale + (HighlightPad * 2 * ResolutionScale) - MiscPosAdjust;
		CellHeight += ClanTagFontIndex >= 0 ? (Fonts[ClanTagFontIndex].CharHeight * FontScale) + (PlayerNamePad * ResolutionScale) - ClanPosAdjust : 0.0f;
		CellHeight += MiscFontIndex >= 0 ? 2*((Fonts[MiscFontIndex].CharHeight * FontScale) + (PlayerNamePad * ResolutionScale)) : 0.0f;
		
		// Check to see if we fit
		if ( CellHeight * PRIList.Length > Canvas.ClipY )
		{
			bRecurse = false;		// By default, don't recurse
			
			if ( FontScale > 0.75 )
			{
				FontScale = FClamp( (Canvas.ClipY/(CellHeight * PRIList.Length)), 0.75f, 1.0f );
				bRecurse = (FontScale <= 0.75);
			}
			else if ( MiscFontIndex > 0 || (ClanTagFontIndex == 0 && MiscFontIndex == 0) )
			{
				// MiscFontIndex is the first to go
				MiscFontIndex--;
				bRecurse = true;
			}
			else if ( ClanTagFontIndex >= 0 )
			{

				// Then the Clan Tag
				ClanTagFontIndex--;
				bRecurse = (ClanTagFontIndex >= 0);
			}
	
			// If we adjusted the ClanTag or Misc sizes, we need to retest the fit.
			if (bRecurse)
			{
				CellHeightHack = AutoFit(GRI, FontIndex, ClanTagFontIndex, MiscFontIndex, FontScale, false);
				return CellHeightHack;
			}
		}

		CellHeight  = Fonts[FontIndex].CharHeight * MainPerc * FontScale + (HighlightPad * 2 * ResolutionScale) - MiscPosAdjust;
		CellHeight += ClanTagFontIndex >= 0 ? (Fonts[ClanTagFontIndex].CharHeight * FontScale) + (PlayerNamePad * ResolutionScale) - ClanPosAdjust : 0.0f;
		CellHeight += MiscFontIndex >= 0 ? 2* ((Fonts[MiscFontIndex].CharHeight * FontScale) + (PlayerNamePad * ResolutionScale)) : 0.0f;
		CellHeightHack = CellHeight;
					
		return CellHeightHack;
	}
	else
		return Super.AutoFit(GRI, FontIndex, ClanTagFontIndex,MiscFontIndex, FontScale, bPrimeList);
}


defaultproperties
{
}


