/**
 * Copyright 2007-2008 Bagotios
 */

class AdvancedUTScoreboardPanel extends UTScoreboardPanel;

// X screen position for the center stat on the panel.
var transient float CenterColumnPosX;
// Width of the header drawn for the center column.
var transient float CenterColumnWidth;
var config bool bDisplayAdditionalInformation;

//Clantag type
var config int CTType;


var string HeaderTitle_Effi;

//Sets the header strings using localized values 
function SetHeaderStrings()
{
	HeaderTitle_Name = Localize( "Scoreboards", "Name", "UTGameUI" );
	HeaderTitle_Score = Localize( "Scoreboards", "Kills", "UTGameUI" );
	HeaderTitle_Deaths = Localize( "Scoreboards", "Deaths", "UTGameUI" );
	HeaderTitle_Effi = "Effi";
}

function PostInitialize()
{
	Super. PostInitialize();
	
	bDisplayAdditionalInformation = (AdvancedUTHUD(UTHudSceneOwner.GetPlayerHud())).bDisplayAdditionalInformation;
	CTType = (AdvancedUTHUD(UTHudSceneOwner.GetPlayerHud())).CTType;
}

/**
 * Update the LRI of a PRI
 */
function DMASUTLPRI GetLRI(UTPlayerReplicationInfo PRI)
{
	//The search should go fast because usually there are not that many CustomPRIs attached
	local UTLinkedReplicationInfo UTLRI;

	if (PRI != None)
	{
		UTLRI = PRI.CustomReplicationInfo;
		while(UTLRI != None)
		{	
			if(UTLRI.IsA('DMASUTLPRI'))
			{
				return (DMASUTLPRI(UTLRI));
			}
			UTLRI = UTLRI.NextReplicationInfo;
		}
	}
	return None;
}

/**
 * Draw the Player's Score
 */
function float DrawScore(UTPlayerReplicationInfo PRI, float YPos, int FontIndex, float FontScale)
{
	local string Spot;
	local float Width, Height;

	// Draw the player's Effi
	Spot = GetPlayerEffi(PRI);
	Canvas.Font = Fonts[FontIndex].Font;
	Canvas.StrLen( Spot, Width, Height );
	DrawString( Spot, RightColumnPosX+RightColumnWidth-Width, YPos,FontIndex,FontScale);
	
	// Draw the player's Deaths
	Spot = GetPlayerDeaths(PRI);
	Canvas.Font = Fonts[FontIndex].Font;
	Canvas.StrLen( Spot, Width, Height );
	DrawString( Spot, CenterColumnPosX+CenterColumnWidth-Width, YPos,FontIndex,FontScale);

	// Draw the player's Frags
	Spot = GetPlayerScore(PRI);
	Canvas.StrLen( Spot, Width, Height );
	DrawString( Spot, LeftColumnPosX+LeftColumnWidth-Width, YPos,FontIndex,FontScale);

	return LeftColumnPosX;
}

/**
 * Draw the Player's Name
 */
function DrawPlayerName(UTPlayerReplicationInfo PRI, float NameOfst, float NameClipX, out float YPos, int FontIndex, float FontScale, bool bIncludeClan)
{

	local float XL, YL;
	local string Spot;
	local Color c ; 
	c = Canvas.DrawColor;
	
	if(PRI.bAdmin)
		Canvas.DrawColor = class'UTHUD'.default.GreenColor;
	Spot = bIncludeClan ? GetPlayerNameStr(PRI) : GetClanTagStr(PRI)$GetPlayerNameStr(PRI);
	StrLen(Spot, XL, YL, FontIndex, FontScale * MainPerc);
	YL = Fonts[FontIndex].CharHeight * FontScale * MainPerc;

	if ( XL > (NameClipX - NameOfst) && !bIncludeClan )
	{
		Spot = GetPlayerNameStr(PRI);
	}

	DrawString( Spot, NameOfst, YPos, FontIndex, FontScale * MainPerc);

	YPos += YL;
	Canvas.DrawColor = c;
}

function DrawScoreHeader()
{
	local float xl,yl, columnWidth, numXL, numYL;

    if ( HeaderFont != none )
    {
		Canvas.SetDrawColor(255,255,255,255);

		Canvas.Font = Fonts[EFT_Large].Font;
		Canvas.StrLen("0000",numXL,numYL);

		Canvas.Font = HeaderFont;
		Canvas.SetPos(Canvas.ClipX * HeaderXPct,Canvas.ClipY * HeaderYPos);
		Canvas.DrawTextClipped(HeaderTitle_Name);

		Canvas.StrLen(HeaderTitle_Effi,xl,yl);
		RightColumnWidth = xl;
		columnWidth = Max(xl+0.25f*numXL, numXL);
		RightColumnPosX = Canvas.ClipX - columnWidth;
		Canvas.SetPos(RightColumnPosX,Canvas.ClipY * HeaderYPos);
		Canvas.DrawTextClipped(HeaderTitle_Effi);
				
		Canvas.StrLen(HeaderTitle_Deaths,xl,yl);
		CenterColumnWidth = xl;
		columnWidth = Max(xl+0.25f*numXL, numXL);
		columnWidth += 0.25f*numXL;
		CenterColumnPosX = Canvas.ClipX - columnWidth;
		CenterColumnPosX = RightColumnPosX - columnWidth;
		Canvas.SetPos(CenterColumnPosX,Canvas.ClipY * HeaderYPos);
		Canvas.DrawTextClipped(HeaderTitle_Deaths);
		
		Canvas.StrLen(HeaderTitle_Score,xl,yl);
		LeftColumnWidth = xl;
		columnWidth = Max(xl, numXL);
		columnWidth += 0.25f*numXL;
		LeftColumnPosX = CenterColumnPosX - columnWidth;
		Canvas.SetPos(LeftColumnPosX, Canvas.ClipY * HeaderYPos);
		Canvas.DrawTextClipped(HeaderTitle_Score);
	}
}

/**
 * Get a PRIs effi as String
 */
function string GetPlayerEffi(UTPlayerReplicationInfo PRI)
{
	local int Efficiency; 
	local DMASUTLPRI LRI;
	LRI = GetLRI(PRI);

	if(LRI.PureKills != 0 && LRI != None)
		Efficiency = (float(LRI.PureKills)/(float(LRI.PureKills)+(float(GetPlayerDeaths(PRI)))))*100;
	else
		Efficiency = 0;
		
	return string(Efficiency);
}

/**
 * Returns the Clan Tag in the PRI
 * @Override
 * remove those annoying brackets
 */
function string GetClanTagStr(UTPlayerReplicationInfo PRI)
{
	local UTGameReplicationInfo GRI;
	if ( PRI != none )
	{
		GRI = UTGameReplicationInfo(PRI.WorldInfo.GRI);
		if ( GRI != none && GRI.bStoryMode )
		{
			if (PRI.bBot || PRI.SinglePlayerCharacterIndex == INDEX_NONE)
			{
				return "";
			}

			return "["$GRI.SinglePlayerBotNames[PRI.SinglePlayerCharacterIndex]$"]";
		}
		if(CTType == 0)
			return PRI.ClanTag != "" ? "[" $(PRI.ClanTag) $ "]" : "";
		else if(CTType == 1)
			return PRI.ClanTag != "" ? (PRI.ClanTag) : "";
		else
			return "";
	}
	else
	{
		return "[Clan]";
	}
}

/**
 * Returns the Player's Name and if the user wants the clantag
 */
function string GetPlayerNameStr(UTPlayerReplicationInfo PRI)
{
	if ( PRI != None )
	{
		if(CTType != 2)
		{
			return PRI.GetPlayerAlias();
		}
		else
			return PRI.ClanTag $ PRI.GetPlayerAlias();
	}
	else
	{
		return FakeNames[NameCnt];
	}
}

/**
 * @Overwrite
 * Add double kills, etc to information
 */
function string GetRightMisc(UTPlayerReplicationInfo PRI)
{
	local string BaseString;
	local DMASUTLPRI LRI;
	LRI = GetLRI(PRI);
		
	if(bDisplayAdditionalInformation)
	{
		if(PRI.WorldInfo.NetMode != NM_Standalone && LRI != None) 
		{
			BaseString = Super.GetRightMisc(PRI);
			
			return "Double: " $ LRI.DoubleKills $ "  Multi: " $ LRI.MultiKills $ "  Mega: " $ LRI.MegaKills $ " Ultra: " $ LRI.UltraKills $ "  Monster: " $ LRI.MonsterKills $ "  " $ BaseString;
		}
		return "";
	}
	else
		return Super.GetRightMisc(PRI);
}

defaultproperties
{
}